require File.expand_path('../helper', __FILE__)

describe Stow do
  it "can instantiate local adapter" do
    storage = Stow.new(:local, base: '/home/tester')
    assert_equal 'Stow::Local', storage.class.name
    assert_equal '/home/tester', storage.base
  end
  
  it "requires and instantiates stow adapters" do
    Stow.stubs(:engines).returns(something: 'Stow::Something')
    klass = mock
    klass.stubs(:new).with(the: 'options').returns('the engine')
    Stow.stubs(:const_get).with('Something').returns(klass)
    Stow.expects(:require).with('stow/something')
    assert_equal 'the engine', Stow.new(:something, the: 'options')
  end
  
  describe "custom storage engines" do
    before do
      @klass = mock
      @klass.stubs(:new).with(my: 'options').returns('the engine')
    end
    
    after do
      Stow.engines.delete(:custom)
    end
    
    it "handles global classes" do
      Object.stubs(:const_get).with('CustomStorageEngine').returns(@klass)      
      Stow.engines[:custom] = 'CustomStorageEngine'
      assert_equal 'the engine', Stow.new(:custom, my: 'options')
    end
    
    it "handles namespaced classes" do
      superclass = mock 
      superclass.stubs(:const_get).with('Engine').returns(@klass)  
      Object.stubs(:const_get).with('CustomStorage').returns(superclass)  
      Stow.engines[:custom] = 'CustomStorage::Engine'
      assert_equal 'the engine', Stow.new(:custom, my: 'options')
    end
    
    it "doesn't try to require custom engines" do
      Stow.stubs(:engines).returns(something: 'CustomEngine')
      Object.stubs(:const_get).with('CustomEngine').returns(@klass)
      Stow.expects(:require).never
      assert_equal 'the engine', Stow.new(:something, my: 'options')
    end
  end
end